package prupp;

import java.io.*;
import java.net.*;
import java.util.*;

/** A class to send files from the htdocs directory.
 *  This makes RabbIT act as a very simple web server.
 */
public class FileSender implements MetaHandler 
{
    /** handler of a MetaPage, that is a special page the proxy supports (like the status page).
     * @param input the InputStream from the client.
     * @param output the OutputStream to the client.
     * @param url the requested url.
     * @param htab the supplied argument to the page (CGI-parameters).
     * @param uap the username and paswword as by Basic-authentication
     */
    public void handle (DataInputStream input, DataOutputStream output, URL url, Properties htab, String uap) {
	String file = htab.getProperty ("argstring");
	if (file == null) 
	    throw (new IllegalArgumentException ("no file given."));
	
	if (file.indexOf ("..") >= 0)    // file is un-url-escaped 
	    throw (new IllegalArgumentException ("Bad filename given"));

	String filename = "htdocs/" + file;
	if (filename.endsWith ("/"))
	    filename = filename + "index.html";
	filename = filename.replace ('/', File.separatorChar);
	HTTPHeader header = Proxy.getHeader ();
	if (filename.endsWith ("gif"))
	    header.setHeader ("Content-type", "image/gif");
	else if (filename.endsWith ("jpeg") || filename.endsWith ("jpg"))
	    header.setHeader ("Content-type", "image/jpeg");
	else if (filename.endsWith ("txt"))
	    header.setHeader ("Content-type", "text/plain");
	
	InputStream fis;
	try {
	    fis = new FileInputStream (filename);
	} catch (IOException e) {
	    throw (new IllegalArgumentException ("could not open file: " + file + "."));	    
	}
	try {
	    output.write (header.toString ().getBytes ());
	    byte buffer[] = new byte[10240];
	    int i;
	    while ((i = fis.read (buffer)) >= 0) {
		output.write (buffer, 0, i);
	    }
	} catch (IOException e) {
	    throw (new IllegalArgumentException ("could not send: " + file + "."));
	}
    }
}
