package prupp;

// $Id$

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.reflect.*;

/** A MetaPage to set a value in the configurations.
 *  This page handles four very distinct sections:
 *  proxy, access, filter and users.
 *  setting is a string either key=value for proxy and filter
 *  or a more general string for access and users.
 */
public class OneSetter implements MetaHandler
{    
    /** default constructor 
     */
    public OneSetter () {    
    }
    
    /** Sets one configuration option in the proxy.
     * @param input the InputStream from the client.
     * @param output the OutputStream to the client.
     * @param url the requested url.
     * @param htab the supplied argument to the page (CGI-parameters).
     * @param uap the username and paswword as by Basic-authentication
     */
    public void handle (DataInputStream input, DataOutputStream output, URL url, Properties htab, String uap) {
	StringBuffer res = new StringBuffer (Proxy.getHeader ().toString ());
	String section = htab.getProperty ("section");
	String filter = htab.getProperty ("filter");
	String setting = htab.getProperty ("setting");
	String key = "";
	String value = "";
	String error = null;

	if (setting != null) {
	    int i;
	    if ((i = setting.indexOf ('=')) >= 0) {
		key = setting.substring (0, i);
		value = setting.substring (i+1);	
	    }
	    else 
		key = setting;
	}
	
	//res.append ("key:" + key + "<BR>value:" + value);
	
	if (section == null || setting == null) {
	    res.append ("Not all arguments supplied");
	}
	else if (section.equals ("proxy")) {
	    Properties pr = new Properties ();
	    pr.put (key, value);
	    Proxy.setup (pr);
	    Proxy.saveConfig ();
	}
	else if (section.equals ("filter")) {
	    if (filter == null) {
		res.append ("you need to specify filter also");
	    }
	    else {
		Properties pr = new Properties ();
		pr.put (key, value);
		
		if (filter.indexOf (".") < 0)
		    filter = "prupp." + filter;
		try {
		    // is this insecure? you can now configure any class....
		    // this is kinda bad or is it good?
		    Class cls = Class.forName (filter);  // throws exception unless found.
		    Class params[] = new Class[1];
		    params[0] = Properties.class;
		    Method setup = cls.getDeclaredMethod ("setup", params);
		    Object arguments[] = new Object [1];
		    arguments[0] = pr;
		    setup.invoke (cls, arguments);	      
		} catch (ClassNotFoundException e) {
		    error = "Couldnt find " + filter;
		} catch (NoSuchMethodException e) {
		    error = "couldnt find method setup for: " + filter + " (this is probably nothing to worry about)";
		} catch (SecurityException e) {
		    error = "Seems reflection aint allowed.";
		} catch (InvocationTargetException e) {
		    error = "couldnt invoke setup for " + filter + 
			", nested exception is:" + e.getTargetException ();
		} catch (IllegalAccessException e) {
		    error = e.toString ();
		}
		if (error != null) {
		    error = "Setup failed: " + error;
		    Proxy.logError (error);
		}
		Proxy.saveFilterConfig ();
	    }
	}
	else if (section.equals ("access")) {
	    try {
		Proxy.readAccess (new StringReader (setting));
	    } catch (IOException e) {
		error = "couldnt set accessess";
	    }
	    Proxy.saveAccess ();
	}
	else if (section.equals ("users")) {
	    try {
		Proxy.readUsers (new StringReader (setting));
	    } catch (IOException e) {
		error = "couldnt set users";
	    }
	    Proxy.saveUsers ();
	}

	res.append ((error != null ? "ERROR: " + error : "OK"));
	try {
	    output.write (res.toString ().getBytes ());
	} catch (IOException e) {
	    Proxy.logError ("Couldnt write status");
	}
    }
}

