package prupp;

// $Id: WebReader.java,v 1.11 1999/01/17 22:01:42 d94-rol Exp $

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;

/** Reads in a page from the web and sends it to the client.
 * Observe that we dont know if we can cache it before we have 
 * read the header.
 */
public class WebReader extends Responder
{
    private boolean dontCache = false;
    private URL url;
    private static SimpleDateFormat sdf = new SimpleDateFormat ("EE, dd MMM yyyy hh:mm:ss");
    
    /**Fetches a page from the net and sends it to the client, caching the page if possible.
     * @param reqLine The request being handled.
     * @param input The inputstream from the client.
     * @param output The outputstream to the client.
     * @param url the page we should get.
     * @param request the full request with headers (and body if any).
     * @param dontFilter if were allowed to filter or not.
     * @param allowCache if were allowed to cache this page.
     */
    public WebReader (String reqLine, 
		      DataInputStream input, 
		      DataOutputStream output, 
		      URL url, 
		      String request, 
		      boolean dontFilter,
		      boolean cacheAllowed) 
	throws IOException {
	super (reqLine, input, output);
	DataOutputStream webos = null;
	DataInputStream webis = null;
	DataOutputStream file = null;
	Socket websocket = null;
	this.url = url;
    
	byte v[] = new byte[10240];
	Proxy.getLogger ().inc ("WebReader(s) created");
	int port = url.getPort () > 0 ? url.getPort () : 80;
	try {
	    InetAddress ia;
	    try {
		ia = Proxy.getInetAddress (url);
	    } catch (UnknownHostException e) {
		Proxy.logError ("Unknown host (" + e.getMessage () + ")");
		error ("couldnt find the specified host(" + e.getMessage () + "), is your URL correct?");
		return;
	    }
	    port = Proxy.getConnectPort (port);
	    
	    try {
		websocket = new Socket (ia, port);	
		websocket.setSoTimeout (5*60*1000);        // 5 min should be enought.
	    } catch (IOException e) {
		error ("couldnt connect to the remote site (" + e.getMessage () + ")");
		return;
	    }
	    webos = new DataOutputStream (websocket.getOutputStream ());
	    webis = new DataInputStream (websocket.getInputStream ());
	    
	    webos.write (request.getBytes());
	
	    // now read the http-header and see if we can cache...
	    HTTPHeader header = null;
	    try {
		header = new HTTPHeader (webis);
	    } catch (IOException e) {
		Proxy.logError ("couldnt read status of page fetched("+url+"): " + e);
		error ("The page you requested is not a valid HTML-page<br>\nReason:"+e);
		return;
	    }
	    
	    String cached = header.getHeader ("pragma");
	    if (cached != null && cached.trim().equals ("no-cache")) {
		Proxy.getLogger().inc ("server says  Pragma no-cache");
		dontCache = true;
	    } 
	    /* dont keep all thoose bad requests, not even authentication ones,
	       and certanly not the 304 (Not modified) */
	    if (header.getStatusline().indexOf (" 200 ") < 0) {
		Proxy.getLogger().inc ("not a: 200 Status ok, better not cache.");
		dontCache = true;
	    }
	    
	    if (dontFilter) { // noproxy-request ?
		dontCache = true;  // yes so dont filter and dont cache
	    }
	    
	    String contentType = header.getHeader ("content-type");
	    if (contentType == null || (contentType.indexOf ("html") < 0 && contentType.indexOf ("javascript") < 0))
		dontFilter = true;
	    
	    int i;
	    if (dontCache) {  // this means that we have a noproxy-request 
		clientos.write (header.toString ().getBytes());
		while ((i = webis.read(v)) > 0) 
		    clientos.write(v,0,i);
		return;
	    }
	    
	    NCacheEntry ent = null;
	    if (cacheAllowed) {
		ent = Proxy.getCache ().newEntry (url);
		Proxy.getLogger ().inc ("Cache entries created");
		
		String expires = header.getHeader ("Expires");
		Date exp = null;
		if (expires != null) {
		    try {
			exp = sdf.parse (expires); 
			ent.setExpires (exp);       // unless we get a parseexception :-).  
		    } catch (ParseException e) {
		    } catch (Exception e) {
			Proxy.logError ("something bad:"+expires+":"+e);
			exp = null;
		    }
		}
	    }
      
	    if (contentType != null && contentType.indexOf ("image") >= 0 && (ent != null)) {
		ImageSender imsend = new ImageSender (clientos, webis, header, ent);
		setExtraInfo ("ImageRatio:" + imsend.getLowqualitySize () + "/" + imsend.getOrigSize () + "=" + imsend.getRatio ());
	    }
	    else {
		if (ent != null) {
		    try {
			file = new DataOutputStream (new FileOutputStream(ent.getFileName ()));
		    } catch (FileNotFoundException e){
			Proxy.logError ("Couldnt find the file " + ent.getFileName ());
			error ("Seems I have a problem...<B>DOH!</B>");
			return;
		    }
		}
		
		if (dontFilter == false) {
		    header.removeHeader ("content-length"); // we must not lie, and chanses are we will if we filter.
		    clientos.write (header.toString().getBytes()); 
		    if (file != null) {
			file.write (header.toString ().getBytes());
		    }
		    Sender send = new Sender (clientos, file, url);	
		    try {
			while ((i = webis.read (v)) > 0) {
			    send.addText (v, i);
			    send.filter ();
			}
			send.cleanup ();
		    }catch (IOException e) {
			Proxy.logError ("Connection closed!");
                        if (file != null) {
                            File f = new File (ent.getFileName ());
                            f.delete ();
                        }
			return;
		    }
		}
		else {
		    clientos.write (header.toString().getBytes()); 
		    if (file != null) {
			file.write (header.toString ().getBytes());
		    }
		    while ((i = webis.read(v)) > 0) {
			clientos.write(v,0,i);
			if (file != null) {
			    file.write (v,0,i);
			}
		    }
		}
	    }
	    Proxy.getCache ().addEntry (ent);
	    Proxy.getLogger ().inc ("Files(s) cached");  
	} catch (IOException e) {	    
	}

	// clean up all resources used.
	finally {
	    if (file != null) {
		try {
		    file.flush ();
		    file.close ();
		}catch (IOException e){}
	    }
	    try {
		if (webos != null){
		    webos.flush ();
		    webos.close ();
		}
		if (webis != null)
		    webis.close ();
		if (websocket != null)
		    websocket.close ();
	    }catch (IOException e){}
	}	
    }  
}
