package prupp.filter;

// $Id$

import java.util.Properties;
import prupp.html.*;

/** How Filters work.
 */
public abstract class Filter
{
    /** Filter a block of text from a page.
     * @param block the HTMLBlock were about to filter.
     * @return the uncertain area of the block.
     */
    public abstract String filter(HTMLBlock block);
  
    /** Called when you want to setup how this filter should work.
     *  Note this is an static method, but "override" it in subclasses as a static
     *  method and it will be called (through reflection).
     * @param properties the properties to use.
     */
    public static void setup (Properties properties) {
	// do nothing, we should not be called. But keep subclasses 
	// from overriding it.
    }
    
    /** Called when you want to get the possible settings this filter 
     *  support. 
     */
    public static Properties getProperties () {
	return new Properties ();
    }

    /** Filters work on a HTMLBlock that contains strings and tags
     *  This function answears the question if it is a string.
     * @param o the object to check.
     */
    public boolean isString (Object o) {
	return (o.getClass() == "".getClass());
    }
}


